#include "model.h"
#include "nodeitem.h"

Model::Model( WObject *parent ) :
  WAbstractItemModel( parent   ),
  root              ( new NodeItem() )
{
}

Model::~Model()
{

}

int Model::columnCount( const WModelIndex &parent ) const
{
  return 1;
}

int Model::rowCount( const WModelIndex &parent ) const
{
  NodeItem *parentItem;
  if (parent.column() > 0)
    return 0;

  if (!parent.isValid())
    parentItem = root.get();
  else
    parentItem = static_cast<NodeItem*>(parent.internalPointer());

  return parentItem->rowCount();
}

boost::any Model::data( const WModelIndex &index, int role ) const
{
  if (!index.isValid())
    return boost::any();


  NodeItem *item = static_cast<NodeItem*>(index.internalPointer());

  if (role == Wt::DisplayRole)
  {
    if( index.column() == 0 )
      return item->sourceIdx;
  }

  return boost::any();
}


WModelIndex Model::index( int row, int column, const WModelIndex &parent ) const
{
  if (!hasIndex(row, column, parent))
    return WModelIndex();

  NodeItem *parentItem;

  if (!parent.isValid())
    parentItem = root.get();
  else
    parentItem = static_cast<NodeItem*>( parent.internalPointer() );

  NodeItem *childItem = parentItem->child(row);
  if (childItem)
    return createIndex(row, column, childItem);
  else
    return WModelIndex();
}

WModelIndex Model::parent(const WModelIndex &index) const
{
  if (!index.isValid())
    return WModelIndex();

  NodeItem *childItem = static_cast<NodeItem*>(index.internalPointer());
  NodeItem *parentItem = childItem->parentItem();

  if (parentItem == root.get())
    return WModelIndex();

  return createIndex(parentItem->row(), 0, parentItem);
}


void Model::addObject( NodeItem *parent )
{
  if( !parent )
  {
    parent = root.get();
  }
  beginInsertRows( parent != root.get() ? createIndex( parent->row(), 0, parent ) : WModelIndex(), parent->rowCount(), parent->rowCount() );
  new NodeItem( parent );
  endInsertRows();
}

void Model::remObject( NodeItem *item )
{
  int row = item->row();
  beginRemoveRows( createIndex( row, 0, item ).parent(), row, row );
  delete item;
  endRemoveRows();
}


